/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#include <TouchWidgets/App.h>
#include "Alert.h"

using namespace TouchWidgets;

void
Alert::create_widgets (void)
{
	if (label || button1) 
		return;

	root_widget = new Widget ();
	label = new Label (message ?: app->translate ("No message."));
	label->set_alignment (TEXT_ALIGN_CENTER);
	label->use_bold_font ();
	button1 = new Button ("OK");
	button1->use_bold_font ();
	button2 = new Button (app->translate ("Cancel"));
	button2->use_bold_font ();
	button1->name = "okButton";
	button2->name = "cancelButton";
	root_widget->add_child (label);
	root_widget->add_child (button1);
	root_widget->add_child (button2);

	root_widget->bg = 0xe0e0e0;
	label->bg = RGB_WHITE;

	button_size = Size_new (72, floorf (72 * .62));

	button1->set_delegate (this);
	button2->set_delegate (this);
}

Alert::Alert (void) 
{
	delegate = NULL;
	message = NULL;
	label = NULL;
	button1 = NULL;
	button2 = NULL;
	create_widgets ();
}

Alert::Alert (const char *str) 
{
	delegate = NULL;
	message = NULL;
	label = NULL;
	button1 = NULL;
	button2 = NULL;
	message = strdup (str);
	create_widgets ();
}

Alert::~Alert ()
{
	if (message)
		free ((void*) message);

	// Destroy the widget tree.
	if (root_widget)
		delete root_widget;

	label = NULL;
	button1 = NULL;
	button2 = NULL;
	root_widget = NULL;
}

void 
Alert::show ()
{
	app->present_popup (this);
}

void 
Alert::layout_widgets (void)
{
	if (!root_widget)
		return;

	int padding = 10;

	Rect r = root_widget->get_bounds ();
	Dimension w = r.size.width;
	Dimension h = r.size.height;

	int available_width = w - 2*padding;
	int available_height = h - 2*padding;
	Dimension h2 = available_height * .62;

	int button_area_width = (available_width - padding) / 2;
	int button_area_height = (available_height - padding) / 2;
	Coordinate button_y = padding + h2 + (button_area_height-button_size.height)/2;
	label->set_rect (Rect_new (padding, padding, available_width, h2));
	button1->set_rect (Rect_new (padding + available_width/3 - button_size.width/2, 
					button_y,
					button_size.width, button_size.height));
	button2->set_rect (Rect_new (padding + 2*available_width/3 - button_size.width/2, 
					button_y,
					button_size.width, button_size.height));
}

void 
Alert::will_appear (void)
{
}

void 
Alert::will_disappear (void)
{
}

void 
Alert::did_appear (void)
{
}

void 
Alert::did_disappear (void)
{
}

void 
Alert::receive_message (Object* sender, const char* message, MessageParameter param)
{
	if (sender == button1) { /* OK */
		if (!strcmp (message, TOUCH_UP_INSIDE)) {
			if (delegate)
				send_message (delegate, POPUP_RESULT, 1);
			app->dismiss_popup ();
		}
	}
	else if (sender == button2) {
		if (!strcmp (message, TOUCH_UP_INSIDE)) {
			if (delegate)
				send_message (delegate, POPUP_RESULT, 0);
			app->dismiss_popup ();
		}
	}
}

