/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_BUTTON_H
#define _TOUCHWIDGETS_BUTTON_H

#include <TouchWidgets/Widget.h>

namespace TouchWidgets {

typedef enum {
	BUTTON_WIDGET_ROUND = 0,
	BUTTON_WIDGET_3D_RECT = 1,
	BUTTON_WIDGET_RECT = 2,
	BUTTON_WIDGET_PLAIN = 3,
} ButtonType;

class Button : public Widget
{
	const char *string;
	int value;

	ButtonType type;
        bool use_bold_default_font;

	bool selected;

	uint32_t button_dark;
	uint32_t button_light2;
	uint32_t button_light;
	uint32_t button_medium2;
	uint32_t button_medium;

	RGB button_selected_color;
	RGB button_selected_fg;
	uint32_t button_selected_dark;
	uint32_t button_selected_light2;
	uint32_t button_selected_light;
	uint32_t button_selected_medium2;
	uint32_t button_selected_medium;

	void setup_colors ();

public:
	void draw (Rect);

	Button (const char *);
	Button (void);
	virtual ~Button (void);

	void set_selected (bool value) { selected = value; redraw (); }

        virtual const char* class_name () { return "Button"; }

        void receive_message (Object *sender, Message, MessageParameter tmp);

        void use_bold_font (void) { use_bold_default_font = true; redraw (); }
};

}

#endif

