/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <math.h>
#include <TouchWidgets/Common.h>
#include <TouchWidgets/App.h>

// using namespace TouchWidgets;

// codereview: Can be improved...
static char string_buffer1 [40];
static char string_buffer2 [40];
static char string_buffer3 [40];

const char *Rect_string (Rect r)
{
	snprintf (string_buffer1, 39, "(%gx%g @ %g,%g)", r.size.width, r.size.height, r.location.x, r.location.y);
	return string_buffer1;
}

const char *Point_string (Point p)
{
	snprintf (string_buffer2, 39, "(%g,%g)", p.x, p.y);
	return string_buffer2;
}

const char *Size_string (Size s)
{
	snprintf (string_buffer3, 39, "(%gx%g)", s.width, s.height);
	return string_buffer3;
}

float Point_distance (Point p, Point p2) 
{
	float dx = (float) (p.x - p2.x);
	float dy = (float) (p.y - p2.y);
	return sqrtf (dx*dx + dy*dy);
}

//-----------------------------------------------------------------------------

static void 
rgb_to_hsv (float r, float g, float b, float *h, float *s, float *v)
{
	// r,g,b values are from 0 to 1
	// h = [0,360], s = [0,1], v = [0,1]
	// if s == 0, then h = -1 (undefined)
	float min, max, delta;
	
	min = MIN3(r, g, b);
	max = MAX3(r, g, b);
	*v = max;
	delta = max - min;
	if (max != 0) {
		*s = delta / max;
	} else {
		*s = 0;
		*h = -1;
		return;
	}
	if (r == max)
		*h = (g - b) / delta;
	else if( g == max )
		*h = 2 + (b - r) / delta;
	else
		*h = 4 + (r - g) / delta;
	*h *= 60;
	if (*h < 0)
		*h += 360;
}

static void 
hsv_to_rgb (float *r, float *g, float *b, float h, float s, float v)
{
	int i;
	float f, p, q, t;
	
	if (s == 0) {
		*r = *g = *b = v;
		return;
	}
	
	h /= 60.f;
	i = floor (h);
	f = h - i;
	p = v * (1 - s);
	q = v * (1 - s * f);
	t = v * (1 - s * (1 - f));
	
	switch (i) {
		case 0:
			*r = v;
			*g = t;
			*b = p;
			break;
		case 1:
			*r = q;
			*g = v;
			*b = p;
			break;
		case 2:
			*r = p;
			*g = v;
			*b = t;
			break;
		case 3:
			*r = p;
			*g = q;
			*b = v;
			break;
		case 4:
			*r = t;
			*g = p;
			*b = v;
			break;
		default:		
		case 5:
			*r = v;
			*g = p;
			*b = q;
			break;
	}
}

RGB
RGB_change_intensity (RGB color, float offset)
{
	long r,g,b;
	r = (color>>16) & 255;
	g = (color >> 8) & 255;
	b = color & 255;

	float red = r / 255.f;
	float green = g / 255.f;
	float blue = b / 255.f;
	float h, s, v;
	rgb_to_hsv (red, green, blue, &h, &s, &v);

	v += offset;
	if (v > 1)
		v = 1;
	hsv_to_rgb (&red, &green, &blue, h, s, v);

	r = red * 255.;
	g = green * 255.;
	b = blue * 255.;
	if (r > 255)
		r = 255;
	if (g > 255)
		g = 255;
	if (b > 255)
		b = 255;
	if (r < 0)
		r = 0;
	if (g < 0)
		g = 0;
	if (b < 0)
		b = 0;
	return (r << 16) | (g << 8) | b;
}


static short shifted_numbers [10] =
{
        ')', '!', '@', '#', '$', '%', '^', '&', '*', '('
};

KeySym
get_shifted_keysym (KeySym key)
{
	if (key == XK_grave)
		key = XK_asciitilde;
	else if (key == ',')
		key = '<';
	else if (key == '.')
		key = '>';
	else if (key == '/')
		key = '?';
	else if (key == '-')
		key = '_';
	else if (key == '=')
		key = '+';
	else if (key == '[')
		key = '{';
	else if (key == ']')
		key = '}';
	else if (key == '\\')
		key = '|';
	else if (key == '\'')
		key = '"';
	else if (key == ';')
		key = ':';
	else if (key >= XK_a && key <= XK_z)
		key = key - XK_a + XK_A;
	else if (key >= XK_0 && key <= XK_9)
		key = shifted_numbers [key - XK_0];

	return key;
}
