/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_CONTAINER_H
#define _TOUCHWIDGETS_CONTAINER_H

#include <TouchWidgets/Widget.h>

namespace TouchWidgets {

class Container : public Widget
{
private:
	Point starting_point; // Site of initial touch.

	bool dont_parse_touches;
	char mode;

protected:
	Size content_size;

public:
	Container ();
	virtual ~Container ();

        virtual const char* class_name () { return "Container"; }

	virtual void destroy_window (void);

	bool container_handles_taps; // Default: false 

	bool get_dont_parse_touches (void);
	void set_dont_parse_touches (bool value);

	void touches_start (Point*, int n_points);
	void touches_continue (Point*, int n_points);
	void touches_end (Point*, int n_points);

	virtual void scroll_start (Point);
	virtual void scroll_continue (Point);
	virtual void scroll_end (Point);

	void detect_content_size (void);
	virtual void set_content_size (Size d);

	virtual void child_was_tapped (int which);
};

}

#endif

