/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_CONTROLLER_H
#define _TOUCHWIDGETS_CONTROLLER_H

#include <TouchWidgets/Common.h>
#include <TouchWidgets/Object.h>

namespace TouchWidgets {

class Widget;

// NOTE: Controller is not descended from Widget. It only controls widgets
// and does not have a window of its own.

class Controller : public Object 
{
protected:
	Widget *root_widget;

	int max_children;
        int n_children;
        Controller **children;
        Controller *next;
        Controller *parent;

        void reallocate_children (int);

public:
	char *title; // XX need setter/getter.

	Size size;

	Controller (void);
	Controller (Widget *root);
	virtual ~Controller (void);

        virtual const char* class_name () { return "Controller"; }

        void add_child (Controller*);
        void insert_child_at (Controller*, int);
        void remove_child (Controller*);
        Controller *get_nth_child (int);

#if 0
	virtual void present_child_at (int n, Rect r);
	virtual void dismiss_child (int n);
#endif

	// virtual void set_root_widget (Widget*);

	void create_windows (void);
	void destroy_windows (void);

	virtual void destroy_widgets (void);
	virtual void layout_widgets (void);

	Widget *get_widget_for_window (Window win);

	void expose_widgets (Rect);
	void create_windows_for_widgets (void);

	virtual void will_appear (void); // Gives controller a chance to prepare data.
	virtual void will_disappear (void); // Gives controller a chance to free up data.
	virtual void did_appear (void); // Gives controller a chance to present sub-controllers.
	virtual void did_disappear (void); // Gives controller a chance to dismiss sub-controllers.

	virtual void set_rect (Rect r);

	//virtual void message (Message *m); // Let subclasses define this.

	Widget *get_root_widget (void);
	Window get_root_window (void);

};

}

#endif
