/*=========================================================================
 * This file is part of TouchTest, which is a demonstration of TouchWidgets.
 *
 * Copyright (C) 2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchTest is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchTest is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TouchTest.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#include <TouchWidgets/Tabs.h>
#include <TouchWidgets/Container.h>
#include <TouchWidgets/Scroll.h>
#include <TouchWidgets/ExampleController.h>

extern "C" {
#include <time.h>
}

using namespace TouchWidgets;

ExampleController::ExampleController (void) 
{
	delegate = NULL;

	// RULE: Controllers need to have widgets created
	// as early as possible so that they will be 
	// "ready for business" even before the time when 
	// the parent controller wants to connect the root widget
	// to its widget tree.
	//
	root_widget = new Widget ();
	root_widget->bg = 0x60c0e0;
	root_widget->name = "ExampleController root";

	button1 = new Button ((char*) "Exit");
	button2 = new Button ((char*) "Animate");

	button1->name = "button1";
	button2->name = "button2";

	button1->set_delegate (this);
	button2->set_delegate (this);

	root_widget->add_child (button1);
	root_widget->add_child (button2);

	container = new Container ();
	container->bg = RGB_YELLOW;
	container->name = "Container for buttons 3,4";
	button3 = new Button ((char*) "Hide/Unhide");
	button4 = new Button ((char*) "Button 4");
	container->add_child (button3);
	container->add_child (button4);

	button3->name = "button3";
	button3->set_delegate (this);

	button4->name = "button4";
	button4->set_delegate (this);

	root_widget->add_child (container);

	canvas = new Canvas (); // Don't know what size will be yet.
	root_widget->add_child (canvas);

	scroll = new Scroll ();
	scroll->name = "Scroll";
	scroll->bg = 0xd0e0f0;
	b1 = new Button ((char*) "Button #1");
	b2 = new Button ((char*) "Button #2");
	b3 = new Button ((char*) "Button #3");
	b4 = new Button ((char*) "Button #4");
	b1->name = "b1";
	b2->name = "b2";
	b3->name = "b3";
	b4->name = "b4";
	scroll->add_child (b1);
	scroll->add_child (b2);
	scroll->add_child (b3);
	scroll->add_child (b4);

	progress1 = new Progress ();
	progress1->set_value (31);
	progress1->fg = 0x4060ff;
	progress1->fg_gradient1 = RGB_change_intensity (progress1->fg, .3);
	progress1->fg_gradient2 = RGB_change_intensity (progress1->fg, -.3);
	root_widget->add_child (progress1);

	textfield = new TextField ();
	textfield->name = "text field";
	textfield->add_string ((char*) "Some text");
	textfield->set_delegate (this);
	root_widget->add_child (textfield);

	img1 = new ImageView ((char*) "NASA.jpg", RGB_BLUE);
	img1->name = "ImageView";
	root_widget->add_child (img1);

	l1 = new Label ((char*) "Label #1");
	l2 = new Label ((char*) "Label #2");
	l3 = new Label ((char*) "Label #3");
	l1->name = "label1";
	l2->name = "label2";
	l3->name = "label3";
	l1->bg = RGB_CYAN;
	l2->bg = RGB_GREEN;
	l3->bg = RGB_BLUE;
	scroll->add_child (l1);
	scroll->add_child (l2);
	scroll->add_child (l3);
	root_widget->add_child (scroll);

	graph1 = new Graph ();
	root_widget->add_child (graph1);

	graph2 = new Graph ();
	root_widget->add_child (graph2);

	graph3 = new Graph ();
	root_widget->add_child (graph3);

	graph1->color = RGB_RED;
	graph1->add_value (25);
	graph1->color = RGB_GREEN;
	graph1->add_value (66);
	graph1->color = RGB_BLUE;
	graph1->add_value (19);
	graph1->color = RGB_ORANGE;
	graph1->add_value (100);

	graph2->set_mode (GRAPH_MODE_LINE);
	graph2->color = RGB_RED;
	graph2->add_value (25);
	graph2->color = RGB_GREEN;
	graph2->add_value (66);
	graph2->color = RGB_BLUE;
	graph2->add_value (19);
	graph2->color = RGB_ORANGE;
	graph2->add_value (100);

	graph3->set_mode (GRAPH_MODE_SCATTER);
	graph3->color = RGB_RED;
	graph3->add_point (Point_new (3,5));
	graph3->add_point (Point_new (16,8.5));
	graph3->add_point (Point_new (11,0));
	graph3->add_point (Point_new (0,6));
	graph3->add_point (Point_new (33,33));

	vlc = new VerticalList ();
	vlc->bg = RGB_STEELBLUE;
	vlc->name = "VLC";
	int i;
	for (i = 0; i < 26; i++) {
		char name [30];
		sprintf (name, "Label %c", i + 'A');
		Label *w = new Label (name);
		w->set_rect (Rect_new (0, 0, 100, 37));
		vlc->add_child (w);
	}
	root_widget->add_child (vlc);

	grid = new Grid ();
	grid->name = "grid";
	l10 = new Label ((char*) "label\n10");
	l11 = new Label ((char*) "label\n11");
	l12 = new Label ((char*) "label 12");
	l13 = new Label ((char*) "label 13");
	l14 = new Label ((char*) "label 14");
	l15 = new Label ((char*) "label 15");
	l16 = new Label ((char*) "label 16");
	b17 = new Button ((char*) "btn 17");
	b17->set_delegate (this);
	l12->set_alignment (TEXT_ALIGN_LEFT);
	l13->set_alignment (TEXT_ALIGN_RIGHT);
	l14->set_alignment (TEXT_ALIGN_CENTER);
	grid->add_child (l10);
	grid->add_child (l11);
	grid->add_child (l12);
	grid->add_child (l13);
	grid->add_child (l14);
	grid->add_child (l15);
	grid->add_child (l16);
	grid->add_child (b17);
	l10->set_rect (Rect_new (0,0,60,37));
	l11->set_rect (Rect_new (0,0,60,37));
	l12->set_rect (Rect_new (0,0,60,37));
	l13->set_rect (Rect_new (0,0,60,37));
	l14->set_rect (Rect_new (0,0,60,37));
	l15->set_rect (Rect_new (0,0,60,37));
	l16->set_rect (Rect_new (0,0,60,37));
	b17->set_rect (Rect_new (0,0,60,37));
	l10->bg = RGB_GRAY;
	l11->bg = RGB_GRAY;
	l12->bg = RGB_GRAY;
	l13->bg = RGB_GRAY;
	l14->bg = RGB_GRAY;
	l15->bg = RGB_GRAY;
	l16->bg = RGB_GRAY;
	b17->bg = RGB_GRAY;
	l10->name = "LABEL 10";
	l11->name = "LABEL 11";
	l12->name = "LABEL 12";
	l13->name = "LABEL 13";
	l14->name = "LABEL 14";
	l15->name = "LABEL 15";
	l16->name = "LABEL 16";
	b17->name = "BUTTON 17";
	grid->set_content_size (Size_new (333, 333));
	root_widget->add_child (grid);

	alert = NULL;
}

void
ExampleController::draw_in_canvas ()
{
	Rect r = canvas->get_bounds ();
	int wid = r.size.width;
	int ht = r.size.height;

	canvas->clear ();

	srand (time (NULL));
	for (int i = 0; i < 20; i++) {
		int x = rand () % wid;
		int y = rand () % ht;
		int w = rand () % wid;
		int h = rand () % ht;
		canvas->fill_rect (Rect_new (x, y, w, h), rand() & 0xffffff);
	}

	canvas->redraw ();
}

void
ExampleController::layout_widgets ()
{
	int padding = 5;
	int x = padding, y = padding;

	Rect r = root_widget->get_rect ();
	int ww = (r.size.width - 5*padding) / 4;
	int hh = (r.size.height - 3*padding - 90) / 2;
	int hh2 = (r.size.height - 4*padding - 90) / 3;

	button1->set_rect (Rect_new (x, y, ww, 40)); y += 40 + padding;
	button2->set_rect (Rect_new (x, y, ww, 40)); y += 40 + padding;
	progress1->set_rect (Rect_new (x, y, ww, 40)); y += 40 + padding;
	textfield->set_rect (Rect_new (x, y, ww, 40)); y += 40 + padding;

	b1->set_rect (Rect_new (10, 10, 100, 50));
	b2->set_rect (Rect_new (10, 70, 100, 50));
	b3->set_rect (Rect_new (10, 130, 100, 50));
	b4->set_rect (Rect_new (10, 190, 100, 50));

	scroll->set_rect (Rect_new (x, y, ww, hh - 40)); y += hh - 40 + padding;

	int remaining_height_first_column = r.size.height - y - padding;
	canvas->setup (ww, remaining_height_first_column, RGB_WHITE);
	canvas->set_rect (Rect_new (x, y, ww, remaining_height_first_column));
	draw_in_canvas ();

	x += ww + padding;
	y = padding;

	vlc->set_rect (Rect_new (x, y, ww/2-padding, hh)); 
	container->set_rect (Rect_new (x+ww/2+padding, y, ww/2-padding, hh));y += hh + padding;
	grid->set_rect (Rect_new (x, y, ww, hh)); y += hh + padding;

	x += ww + padding;
	y = padding;

	img1->set_rect (Rect_new (x, y, ww, 2*hh+padding)); y += hh + padding;

	button3->set_rect (Rect_new (padding, padding, 80, 50));
	button4->set_rect (Rect_new (padding, 50+2*padding, 80, 50));
	l1->set_rect (Rect_new (5, 250, 100, 40));
	l2->set_rect (Rect_new (10, 290, 100, 40));
	l3->set_rect (Rect_new (15, 330, 100, 40));

	y = padding;
	x += ww + padding;
	graph1->set_rect (Rect_new (x, y, ww, hh2));
	y += hh2 + padding;
	graph2->set_rect (Rect_new (x, y, ww, hh2));
	y += hh2 + padding;
	graph3->set_rect (Rect_new (x, y, ww, hh2));

	scroll->detect_content_size ();
	vlc->reposition_children ();
}

void
ExampleController::will_appear ()
{
	// Prepare data here.
}

void
ExampleController::will_disappear ()
{
	// Deallocate data here.
}

void
ExampleController::did_appear ()
{
	// Called after widgets' windows created.
}

void
ExampleController::did_disappear ()
{
	// Called after widgets' windows destroyed.
}

static int foo = 0;
static int foo2 = 0;

void 
ExampleController::receive_message (Object *sender, Message m, MessageParameter p)
{
	printf ("ExampleController received an incoming Message: %s from: %s\n", m, sender->name);

	if (sender == button1)
		app->shutdown ();
	else
	if (sender == button2) {
		Tabs *super = (Tabs*) parent;
		super->set_title ("This is a test.");

		if (!foo) {
			container->animate_to_rect (Rect_new (300,300,300,300), 2);
			foo=1;
		} else {
			container->animate_to_rect (Rect_new (300,10,300,200), 1);
			foo=0;
		}
	}
	else
	if (sender == button3) {
		button4->set_hidden (!button4->get_hidden ());
	}
	else
	if (sender == button4) {
		int value = progress1->get_value () + 10;
		if (value > 100) 
			value = 0;
		progress1->set_value (value);

		alert = new Alert ("This is a test of the Alert class\nand of the two-line capability of Label.");
		alert->name = "alertController";
		alert->set_delegate (this);
		alert->show ();
	}
	else
	if (sender == textfield) {
		if (!strcmp (m, DONE_EDITING)) {
			printf ("Done editing, line = %s\n", textfield->get_text ());
		}
	}
	else
	if (sender == b17) {
		draw_in_canvas ();
	}
	else
	if (sender == alert) { // !strcmp (m, POPUP_RESULT)) {
		printf ("Popup result: %d\n", (int) p);
		// alert is already deallocated.
		alert = NULL;
	}
}

