/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <TouchWidgets/Grid.h>

using namespace TouchWidgets;

Grid::Grid (void) 
{
	delegate = NULL;
	n_rows = 3;
	n_columns = 3;
	n_pages = 1;
	show_separators = 1;
	padding = 10;
}

Grid::~Grid (void) 
{

}

void
Grid::child_was_tapped (int which)
{
        printf ("Child %u was tapped.\n", which);

        if (delegate) {
                send_message (delegate, TOUCH_UP_INSIDE);
        }
}

void
Grid::reposition_children (void)
{
        if (!n_children)
                return;

	if (n_columns < 1)
		n_columns = 1;
	if (n_rows < 1)
		n_rows = 1;

        Dimension width = rect.size.width;
        Dimension height = rect.size.height;
	Dimension cell_width = width / (float) n_columns;
	Dimension cell_height = height / (float) n_rows;

        for (int i = 0; i < n_children; i++) {
		int col = i % n_columns;
		int row = i / n_columns;

		// Determine center of cell.
		Coordinate cx = cell_width * (col + .5);
		Coordinate cy = cell_height * (row + .5);

                Widget *widget = children[i];
                Rect r = widget->get_rect ();
                Dimension w = r.size.width;
                Dimension h = r.size.height;
		cx -= w/2.;
		cy -= h/2.;
                Rect new_rect = Rect_new (cx, cy, w, h);
		widget->set_rect (new_rect);
        }

	n_pages = (n_children + n_columns * n_rows - 1) / (n_columns * n_rows);
	Size size = Size_new (width * n_pages, height);
	set_content_size (size);
}

void
Grid::draw (Rect)
{
	reposition_children ();
}

