/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <stdlib.h>

#include <TouchWidgets/App.h>
#include <TouchWidgets/Label.h>

using namespace TouchWidgets;

Label::Label () 
{
	string = NULL;
	align = TEXT_ALIGN_LEFT;
	use_bold_default_font = false;
	rect = Rect_zero ();
	border = 0;
}

Label::Label (const char *s) 
{
	string = NULL;
	align = TEXT_ALIGN_LEFT;
	use_bold_default_font = false;
	rect = Rect_zero ();
	border = 0;

	if (!s)
		s = (char*) "";

	string = strdup (s);
}

Label::~Label ()
{
	if (string) {
		free ((void*) string);
		string = NULL;
	}
}

void
Label::draw (Rect r)
{
	if (!window || hidden)
		return;

	XGCValues gcv;
	memset ((void*) &gcv, 0, sizeof(XGCValues));
	GC gc = XCreateGC (app->dpy, window, 0, &gcv);

	clip_into (r, gc);

	XFontStruct *font = !use_bold_default_font ? app->default_font : app->default_font_bold;
	XSetFont (app->dpy, gc, font->fid);

	Dimension w = rect.size.width;
	Dimension h = rect.size.height;

	XSetForeground (app->dpy, gc, bg);
	XFillRectangle (app->dpy, window, gc, 0, 0, w, h);

	int x = border; // Left aligned

	int len = strlen (string);

	XSetForeground (app->dpy, gc, fg);
	Dimension ascent = font->ascent;

	// Get the text width(s) for up to 2 lines.
	//
	Dimension tw = 0;
	Dimension tw2 = 0;
	char *s2 = strchr ((char*) string, '\n');
	if (!s2) {
		tw = XTextWidth (font, string, len);
		switch (align) {
			case TEXT_ALIGN_RIGHT: 
				x += w - tw;
				x -= 2*border;
				break;
			case TEXT_ALIGN_CENTER: 
				x = border + ((w-2*border) - tw)/2;
				break;
		}

		int y = ascent + border + ((h-2*border) - ascent)/2;
		XDrawString (app->dpy, window, gc, x, y, string, len);
	} else {
		len = s2 - string;
		tw = XTextWidth (font, string, s2 - string);
		s2++;
		int len2 = strlen (s2);
		tw2 = XTextWidth (font, s2, strlen (s2));

		switch (align) {
			case TEXT_ALIGN_RIGHT: 
				x += w - tw;
				x -= 2*border;
				break;
			case TEXT_ALIGN_CENTER: 
				x = border + ((w-2*border) - tw)/2;
				break;
		}

		Coordinate x2 = border;
		switch (align) {
			case TEXT_ALIGN_RIGHT: 
				x2 += w - tw2;
				x2 -= 2*border;
				break;
			case TEXT_ALIGN_CENTER: 
				x2 = border + ((w-2*border) - tw2)/2;
				break;
		}

		// Draw the upper line.
		int line_height = ascent + font->descent;
		int y = border + (h-2*border)/2 - line_height + ascent;
		XDrawString (app->dpy, window, gc, x, y, string, len);

		// Draw the lower line.
		y = border + (h-2*border)/2 + ascent;
		XDrawString (app->dpy, window, gc, x2, y, s2, len2);
	}

	XFreeGC (app->dpy, gc);
}

