/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_MESSAGE_H
#define _TOUCHWIDGETS_MESSAGE_H

#include <TouchWidgets/Common.h>

namespace TouchWidgets {

typedef const char* Message;
typedef uint32_t MessageParameter;

static inline MessageParameter MessageParameter_from_point (Point p)
{
	typedef union {
		int16_t i;
		uint16_t u;
	} U;
	U u1, u2;
	u1.i = (int16_t) p.x;
	u2.i = (int16_t) p.y;
	uint32_t result = (u1.u << 16) | u2.u;
	return result;
}

static inline Point Point_from_message (MessageParameter m)
{
	typedef union {
		int16_t i;
		uint16_t u;
	} U;
	U u1, u2;
	u1.u = 0xffff & (m >> 16);
	u2.u = 0xffff & m;
	return Point_new ((Coordinate) u1.i, (Coordinate) u2.i);
}

#define TIMER ("timer")
#define TOUCH_UP_INSIDE ("touchUpInside")
#define TOUCH_UP_OUTSIDE ("touchUpOutside")
#define TOUCH_DOWN_INSIDE ("touchDownInside")
#define TOUCH_DOWN_OUTSIDE ("touchDownOutside")
#define KEY_DOWN ("keyDown")
#define KEY_UP ("keyUp")
#define DONE_EDITING ("doneEditing")
#define BUTTONBAR_SELECTION ("buttonBarSelection")
#define FINISHED_ANIMATING ("finishedAnimating")
#define POPUP_RESULT ("popupResult")
#define DISMISS_KEYBOARD ("dismissKeyboard")

}

#endif
