/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_OBJECT_H
#define _TOUCHWIDGETS_OBJECT_H

#include <TouchWidgets/Common.h>
#include <TouchWidgets/Message.h>

namespace TouchWidgets {

// The purpose of this class is to define the most basic object
// within the context of the touch interface, and to facilitate
// sending and receiving of messages between these objects.
 
class Object
{
protected:
	const char *style_class;// For style fetching; example: screen2_labels
	Object *delegate;

public:
	Object (void);
	~Object (void);

	virtual const char* class_name () { return "Object"; }

	const char *name;	// Dual purpose: Serves as style id, and used for inter-object messaging.

	void set_delegate (Object *d);
	Object *get_delegate (void) { return delegate; }

	void set_style_class (const char *s_) { style_class = s_; }

	void send_message (Object *recipient, Message m) {
		if (recipient) {
			recipient->receive_message (this, m, 0);
		} 
	}

	void send_message (Object *recipient, Message m, MessageParameter p) {
		if (recipient) {
			recipient->receive_message (this, m, p);
		} 
	}

	virtual void receive_message (Object *sender, Message m, MessageParameter p);
};

}

#endif
