/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <TouchWidgets/App.h>
#include <TouchWidgets/Progress.h>

using namespace TouchWidgets;

void
Progress::draw (Rect clip)
{
        if (!window || hidden)
                return;

	XGCValues gcv;
	memset ((void*) &gcv, 0, sizeof(XGCValues));
	GC gc = XCreateGC (app->dpy, window, 0, &gcv);

	clip_into (clip, gc);

	int w = (int) rect.size.width;
	int h = (int) rect.size.height;

	if (value<0)
		value=0;
	else if (value>100)
		value=100;

	if (border) {
		XSetForeground (app->dpy, gc, border_color);
		XDrawRectangle (app->dpy, window, gc, 0, 0, w-1, h-1);
	}

	XSetForeground (app->dpy, gc, bg);
	XFillRectangle (app->dpy, window, gc, border, border,w-2*border,h-2*border);

	if (value > 0) {
		RGB tmp1 = bg;
		RGB tmp2 = bg_gradient;

		Rect r = rect;
		r.location.x = border;
		r.location.y = border;
		r.size.width -= 2*border;
		r.size.height -= 2*border;

		if (value < 100) {
			uint32_t span = w - border*2;
			span *= value;
			span /= 100;
			r.size.width = span;
		}

		// Upper 1/4 transitions from fg to fg_gradient1.
		Dimension htmp = r.size.height;
		r.size.height *= .25;
		bg = fg;
		bg_gradient = fg_gradient1;
		draw_gradient_in_rect (clip, r);
		bg = tmp1;
		bg_gradient = tmp2;

		// Lower 3/4 transitions from fg_gradient1 to fg_gradient2.
		r.location.y += r.size.height;
		r.size.height = htmp - r.size.height + 1;
		bg = fg_gradient1;
		bg_gradient = fg_gradient2;
		draw_gradient_in_rect (clip, r);
		bg = tmp1;
		bg_gradient = tmp2;

	}

	XFreeGC (app->dpy, gc);
}

Progress::Progress () 
{
	delegate = NULL;
	value = 0;
	fg_gradient1 = RGB_change_intensity (fg, .3);
	fg_gradient2 = RGB_change_intensity (fg, -.3);
	border = 1;
}

void
Progress::set_value (int value_)
{
	value = value_;
	redraw ();
}

