/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_SCROLL_CONTAINER_H
#define _TOUCHWIDGETS_SCROLL_CONTAINER_H

#include <TouchWidgets/Container.h>

namespace TouchWidgets {

#define TOUCH_SPEED_THRESHOLD_FOR_MOMENTUM (.05)

class Scroll : public Container
{
	Point content_offset;
	Point additional_offset_during_touches;
	bool allow_horizontal_scrolling;
	bool allow_vertical_scrolling;
	bool scrolling;
        Point *children_original_locations;
	Window moveable_window;

	void limit_scrolling (Point *p);

	int internal_animation_counter;
	Coordinate internal_animation_delta_x;
	Coordinate internal_animation_delta_y;
	Point internal_animation_point;
	void begin_internal_animation (Point vector);
	void end_internal_animation (void);

public:
	Scroll ();
	~Scroll ();

        virtual const char* class_name () { return "Scroll"; }

	Point get_content_offset (void);

	bool use_momentum;

	void scroll_to (Point p);

	Window get_window (void); // Provide the moveable window to childen.

        virtual void scroll_start (Point);
        virtual void scroll_continue (Point);
        virtual void scroll_end (Point);

	void perform_internal_animation (void);

	void set_content_size (Size size);

	virtual void destroy_window (void);

	virtual void child_was_tapped (int which);
};

}

#endif

