/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <TouchWidgets/App.h>
#include <TouchWidgets/StatusBar.h>

using namespace TouchWidgets;

void
StatusBar::draw (Rect r)
{
        if (!window || hidden)
                return;

	char *s;

	XGCValues gcv;
	memset ((void*) &gcv, 0, sizeof(XGCValues));
	GC gc = XCreateGC (app->dpy, window, 0, &gcv);

	int w, h;
	w = rect.size.width;
	h = rect.size.height;
	s = string;

	XSetFont (app->dpy, gc, app->default_font->fid);
	XSetForeground (app->dpy, gc, button_medium);
	XFillRectangle (app->dpy, window, gc, border, border, w-2*border, h-2*border);

	int i;
	for (i=0; i<border; i++)
	{
		XSetForeground (app->dpy, gc, button_dark);
		XDrawLine (app->dpy, window, gc, i,i,i,h-1-i);
		XDrawLine (app->dpy, window, gc, i,i,w-1-i,i);

		XSetForeground (app->dpy, gc, button_light2);
		XDrawLine (app->dpy, window, gc, w-1-i,1+i,w-1-i,h-1-i);
		XDrawLine (app->dpy, window, gc, i,h-1-i,w-1-i,h-1-i);
	}

	// int tw = XTextWidth (app->default_font, s, strlen(s));
	int ascent = app->default_font->ascent;
	// int th = ascent + app->default_font->descent;

	XSetForeground (app->dpy, gc, fg);
	XDrawString (app->dpy, window, gc, border + padding,
		padding + ascent + border,
		s, strlen(s));

	XFreeGC (app->dpy, gc);
}

StatusBar::StatusBar () 
{
	delegate = NULL;
	string = NULL;
	padding = 3;
	border = 2;
}

StatusBar::StatusBar (RGB fg_, RGB bg_, char *s) 
{
	delegate = NULL;
	string = NULL;
	padding = 3;
	border = 2;

	fg = fg_;
	bg = bg_;

	int height = app->default_font->ascent + app->default_font->descent + 2*border + 2*padding;
	rect.size.height = height;

	if (!s)
		s = (char*) "";

	string = strdup (s);
}

