/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_TAB_CONTROLLER_H
#define _TOUCHWIDGETS_TAB_CONTROLLER_H

#include <TouchWidgets/Controller.h>

namespace TouchWidgets {

#define DEFAULT_SLIDE_TIME 0.6

class TitleBar;
class ButtonBar;

// This controller has a root widget that is a big blank background.
// It has 3 children:
//	- The title bar.
//	- The content that is provided by the app, which is whatever controller's root widget that is visible at the moment.
// 	- The button bar.

class Tabs : public Controller
{
	TitleBar *title_bar;
	Widget *current_content_root;
	ButtonBar *button_bar;
	int selection;
	
	// For animation:
	int animations_count;
	Widget *incoming_content_root; 
	Widget *outgoing_content_root; 
	int incoming_controller;
	int outgoing_controller;

	void select_nth_instantaneous (int);
	void select_nth_animated (int);

public:
	Tabs ();
	~Tabs ();

	void set_title (const char*);

	void select_nth (int n);

	bool use_animation;
	float animation_duration;

        virtual const char* class_name () { return "Tabs"; }

        void destroy_widgets (void);
        void layout_widgets (void);

	void will_appear (void);
	void will_disappear (void);
	void did_appear (void);
	void did_disappear (void) { }

	void deselect (void);

	void add_button_and_child (const char *, Controller *);
	void remove_button_and_child (int);

        void receive_message (Object *sender, Message, MessageParameter tmp);
};

}

#endif

