/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#ifndef _TOUCHWIDGETS_TEXTFIELD_H
#define _TOUCHWIDGETS_TEXTFIELD_H

#include <TouchWidgets/Widget.h>

namespace TouchWidgets {

class TextField : public Widget
{
#define TEXTENTRY_BORDER (4)
#define TEXTENTRY_PADDING (4)   
#define TEXTENTRY_MAXLEN (256)

protected:
        uint8_t char_positions [TEXTENTRY_MAXLEN];
        int length;
        int cursor_position;
        bool full;
        bool have_focus;

	void key_press (KeySym);
	void key_release (KeySym);
	void focus_taken (void);

	bool ctrl, shift;

        char line [TEXTENTRY_MAXLEN];

public:
	void erase (void);
	void add_char (char ch);
	void add_integer (int);
	void add_string (char *str);
	char *get_text (void);

	virtual void draw (Rect);

	TextField ();
	TextField (char *);
	virtual ~TextField ();

        virtual const char* class_name () { return "TextField"; }

	void receive_message (Object *sender, Message msg, MessageParameter param); // Inherited.

};

}

#endif

