/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <stdlib.h>

#include <TouchWidgets/App.h>
#include <TouchWidgets/TitleBar.h>

using namespace TouchWidgets;

TitleBar::TitleBar () :
	string(NULL)
{
	delegate = NULL;
	rect = Rect_zero ();
	bg = RGB_BLUE;
	fg = RGB_WHITE;
}

TitleBar::TitleBar (const char *s) 
{
	delegate = NULL;
	rect = Rect_zero ();
	bg = RGB_BLUE;
	fg = RGB_WHITE;

	if (!s)
		s = app->translate ("Untitled");

	string = strdup (s);
}

TitleBar::~TitleBar ()
{
	if (string) {
		free (string);
		string = NULL;
	}
}

void
TitleBar::draw (Rect clip)
{
	if (!window || hidden)
		return;

	XGCValues gcv;
	memset ((void*) &gcv, 0, sizeof(XGCValues));
	GC gc = XCreateGC (app->dpy, window, 0, &gcv);

	clip_into (clip, gc);

	Dimension w = rect.size.width;
	Dimension h = rect.size.height;

	bg_gradient = RGB_change_intensity (bg, -.3);
	draw_gradient_background (clip);

	XFontStruct *font = app->default_title_font_bold;
	if (!font) 
			return;
	XSetFont (app->dpy, gc, font->fid);

	char *s = string;
	XSetForeground (app->dpy, gc, fg);
	Dimension tw = XTextWidth (font, s, strlen(s));
	Dimension ascent = font->ascent;
	Dimension th = ascent + font->descent;

	int x = border;
#define TWEAK 2
	int y = ascent + (h - border - th)/2 + TWEAK;

	XDrawString (app->dpy, window, gc, x + border + ((w-2*border) - tw)/2,
		y,
		s, strlen(s));

	XFreeGC (app->dpy, gc);
}

