/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#include <stdbool.h>
#include <string.h>
#include <locale.h>

#include <TouchWidgets/Util.h>

int get_visual_class (XVisualInfo *vinfo)
{
	if (vinfo)
		return vinfo->class;
	else
		return -1;
}

void
bresenham_init (BresenhamInfo *info, int target_size, int original_size)
{
	if (!info) 
		return;
	if (!target_size || !original_size)
		return;
	if (target_size > original_size)
		return;

	info->dx = original_size;
	info->dy = target_size;
	info->e = (info->dy<<1) - info->dx;
	info->j = 0;
	info->sum = 0;
}

int
bresenham_get_next (BresenhamInfo *info)
{
	if (!info) 
		return 0;

	char done = 0;

	done = false;

	int initial_j = info->j;

	while (info->j <= info->dx && !done)
	{
		if (info->e >= 0)
		{
			done = true;
			info->e -= (info->dx<<1);
		}
		info->e += (info->dy<<1);
		++info->j;
	}

	int count = info->j - initial_j;
	info->sum += count;
	if (info->sum > info->dx)
	{
		int *i=0;
		*i++;
	}
	return count;
}

