/*=========================================================================
 * This file is part of TouchWidgets, which is a widget library that
 * supports touch and animation.
 *
 * Copyright (C) 2004,2008-2010,2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchWidgets is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchWidgets is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with TouchWidgets.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/


#include <TouchWidgets/VerticalList.h>

using namespace TouchWidgets;

VerticalList::VerticalList (void) 
{
	delegate = NULL;
	container_handles_taps = true;
}

void
VerticalList::child_was_tapped (int which)
{
#ifdef DEBUG_VLC
	printf ("Child %u was tapped.\n", which);
#endif

	if (delegate) {
		send_message (delegate, TOUCH_UP_INSIDE);
	}
}

void
VerticalList::draw (Rect r)
{
	reposition_children ();
}

void
VerticalList::reposition_children (void)
{
	if (!n_children)
		return;

	Coordinate y = 0;
	Dimension width = rect.size.width;
	for (int i = 0; i < n_children; i++) {
		Widget *w = children[i];
		Rect r = w->get_rect ();
		Dimension h = r.size.height;
		Rect new_rect = Rect_new (0, y, width, h);
		if (!Rect_equal (r, new_rect))
			w->set_rect (new_rect);
		y += h;

		if (i < n_children-1)
			y++;
	}

	set_content_size (Size_new (width, y));
}


