/*=========================================================================
 * This file is part of TouchTest, which is a demonstration of TouchWidgets.
 *
 * Copyright (C) 2013 Zachary Smith, veritas@comcast.net.
 *
 * TouchTest is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TouchTest is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TouchTest.  If not, see <http://www.gnu.org/licenses/>.
 *=======================================================================*/

#include <TouchWidgets/App.h>
#include <TouchWidgets/Controller.h>
#include <TouchWidgets/Button.h>
#include <TouchWidgets/ImageView.h>
#include <TouchWidgets/Tabs.h>

#include "ExampleController.h"

using namespace TouchWidgets;

int main (int argc, char **argv)
{
	app = new App ();

	app->initialize (argc, argv, (char*)"Test App", true, 1000, 600, RGB_WHITE, 10, RGB_BLACK);

	int i;

	//------------------------------
	// Create the root controller.
	//
        Tabs *tab_controller = new Tabs ();
	tab_controller->name = "Tab controller";
//	tab_controller->use_animation = false;
	tab_controller->set_title ("To enable the mouse pointer, run with the -cursor command-line option.");

	//----------------------------------------
        // Create a child controller and connect 
	// it to the widget tree.
	//
	static const char *titles [] = {
		 "Example",
		 "Second",
		 "3rd",
		 "Fourth",
		 "Fifth",
	};

	ExampleController *ctl = new ExampleController ();
	tab_controller->add_button_and_child (NULL, ctl); // Leave out name.

	//------------------------------
	// Create an image view tab.
	//
        Controller *c;
	ImageView *iv;
	iv = new ImageView ((char*) "NASA.jpg", RGB_YELLOW);
//	iv->set_mode (WIDGET_ALIGN_TOP_LEFT);
	iv->name = "IV";
//	iv->set_backing_store (true);
	c = new Controller (iv);
	c->name = "touch controller for solo ImageView";
	tab_controller->add_button_and_child ("Image", c);

	//------------------------------
        // Now give the tab controller 
	// four more children.
	//
        Button *b;
        for (i = 1; i < 4; i++) {
                char foo [20];
                sprintf (foo, "btn %d", i);
                b = new Button (foo);
		b->name = strdup (foo);
                c = new Controller (b);
		c->name = "touch controller for solo button";

                tab_controller->add_button_and_child (titles [i], c);
        }

	app->push_window (WINDOW_MAIN, tab_controller, 0, 0, 0xff, "Test App");

	//---------------
	// Main loop.
	//
	app->run ();

	delete app;

	return 0;
}

